@@ -26,15 +26,15 @@ Enables identical use for installed and uninstalled versions.
 import os.path
 import string
 
-LIBDIR = '/usr/local/lib'
-PKGDATADIR = '/usr/local/share/pitivi'
-PIXMAPDIR = '/usr/local/share/pixmaps'
-pitivi_version = '0.10.3'
+LIBDIR = '/usr/lib'
+PKGDATADIR = '/usr/share/pitivi'
+PIXMAPDIR = '/usr/share/pixmaps'
+pitivi_version = '0.11.1'
 APPNAME = 'PiTiVi'
 PYGTK_REQ = '2.8.0'
 PYGST_REQ = '0.10.6'
-GST_REQ = '0.10.8'
-GNONLIN_REQ = '0.10.6.1'
+GST_REQ = '0.10.13.1'
+GNONLIN_REQ = '0.10.8.1'
 PYCAIRO_REQ = '1.0.0'
 
 def get_pixmap_dir():
