@@ -26,16 +26,18 @@ Providers of elements to use in a timeline
 
 import os.path
 from urllib import unquote
+import weakref
+from random import randint
 import string
 import gobject
 import gst
 
-import utils
+from serializable import Serializable
 from settings import ExportSettings
 
 from gettext import gettext as _
 
-class ObjectFactory(gobject.GObject):
+class ObjectFactory(gobject.GObject, Serializable):
     """
     base class for object factories which provide elements to use
     in the timeline
@@ -45,7 +47,7 @@ class ObjectFactory(gobject.GObject):
                        "Contains audio stream",
                        "Does the element contain audio",
                        False, gobject.PARAM_READWRITE),
-        
+
         "is-video" : ( gobject.TYPE_BOOLEAN,
                        "Contains video stream",
                        "Does the element contain video",
@@ -60,7 +62,16 @@ class ObjectFactory(gobject.GObject):
                          gobject.PARAM_READWRITE )
         }
 
-    def __init__(self):
+    __data_type__ = "object-factory"
+
+    # UID (int) => object (BrotherObjects) mapping.
+    __instances__ = weakref.WeakValueDictionary()
+
+    # dictionnary of objects waiting for pending objects for completion
+    # pending UID (int) => objects (list of BrotherObjects and extra field)
+    __waiting_for_pending_objects__ = {}
+
+    def __init__(self, **kwargs):
         gobject.GObject.__init__(self)
         self.name = ""
         self.displayname = ""
@@ -75,6 +86,7 @@ class ObjectFactory(gobject.GObject):
         self.mediaTags = {}
         self.title = None
         self.artist = None
+        self.uid = -1
 
     def do_set_property(self, property, value):
         """
@@ -155,10 +167,6 @@ class ObjectFactory(gobject.GObject):
             else:
                 stl.append(_("<b>%s</b>") % gobject.markup_escape_text(self.title))
             stl.append(_("<small><b>File:</b> %s") % filename)
-##         if self.title:
-##             stl.append("<b>Title:</b> %s" % self.title)
-##         if self.artist:
-##             stl.append("<b>Artist:</b> %s" % self.artist)
         if self.is_video and self.video_info_stream:
             stl.append(self.video_info_stream.getMarkup())
         if self.is_audio and self.audio_info_stream:
@@ -174,6 +182,94 @@ class ObjectFactory(gobject.GObject):
         raise NotImplementedError
 
 
+    # Serializable methods
+
+    def toDataFormat(self):
+        ret = Serializable.toDataFormat(self)
+        ret["uid"] = self.getUniqueID()
+        ret["name"] = self.name
+        ret["displayname"] = self.displayname
+        ret["is_audio"] = self.is_audio
+        ret["is_video"] = self.is_video
+        return ret
+
+    def fromDataFormat(self, obj):
+        Serializable.fromDataFormat(self, obj)
+        self.name = obj["name"]
+        self.displayname = obj["displayname"]
+        self.is_audio = obj["is_audio"]
+        self.is_video = obj["is_video"]
+        self.setUniqueID(obj["uid"])
+
+    # Unique ID methods
+
+    def getUniqueID(self):
+        if self.uid == -1:
+            i = randint(0, 2**32)
+            while i in ObjectFactory.__instances__:
+                i = randint(0, 2 ** 32)
+            self.uid = i
+            ObjectFactory.__instances__[self.uid] = self
+        return self.uid
+
+    def setUniqueID(self, uid):
+        if not self.uid == -1:
+            gst.warning("Trying to set uid [%d] on an object that already has one [%d]" % (uid, self.uid))
+            return
+
+        if uid in ObjectFactory.__instances__:
+            gst.warning("Uid [%d] is already in use by another object [%r]" % (uid, ObjectFactory.__instances__[uid]))
+            return
+
+        self.uid = uid
+        gst.log("Recording __instances__[uid:%d] = %r" % (self.uid, self))
+        ObjectFactory.__instances__[self.uid] = self
+
+        # Check if an object needs to be informed of our creation
+        self._haveNewID(self.uid)
+
+    @classmethod
+    def getObjectByUID(cls, uid):
+        """
+        Returns the object with the given uid if it exists.
+        Returns None if no object with the given uid exist.
+        """
+        if uid in cls.__instances__:
+            return cls.__instances__[uid]
+        return None
+
+    # Delayed object creation methods
+
+    def _haveNewID(self, uid):
+        """
+        This method is called when an object gets a new ID.
+        It will check to see if any object needs to be informed of the creation
+        of this object.
+        """
+        if uid in ObjectFactory.__waiting_for_pending_objects__ and uid in ObjectFactory.__instances__:
+            for obj, extra in ObjectFactory.__waiting_for_pending_objects__[uid]:
+                obj.pendingObjectCreated(ObjectFactory.__instances__[uid], extra)
+            del ObjectFactory.__waiting_for_pending_objects__[uid]
+
+
+    @classmethod
+    def addPendingObjectRequest(cls, obj, uid, extra=None):
+        """
+        Ask to be called when the object with the given uid is created.
+        obj : calling object
+        uid : uid of the object we need to be informed of creation
+        extra : extradata with which obj's callback will be called
+
+        The class will call the calling object's when the requested object
+        is available using the following method call:
+        obj.pendingObjectCreated(new_object, extra)
+        """
+        if not uid in cls.__waiting_for_pending_objects__:
+            cls.__waiting_for_pending_objects__[uid] = []
+        cls.__waiting_for_pending_objects__[uid].append((weakref.proxy(obj), extra))
+
+gobject.type_register(ObjectFactory)
+
 class FileSourceFactory(ObjectFactory):
     """
     Provides File sources useable in a timeline
@@ -194,9 +290,11 @@ class FileSourceFactory(ObjectFactory):
                          gobject.PARAM_READWRITE )
         }
 
-    def __init__(self, filename, project):
+    __data_type__ = "file-source-factory"
+
+    def __init__(self, filename="", project=None, **kwargs):
         gst.info("filename:%s , project:%s" % (filename, project))
-        ObjectFactory.__init__(self)
+        ObjectFactory.__init__(self, **kwargs)
         self.project = project
         self.name = filename
         self.displayname = os.path.basename(unquote(self.name))
@@ -204,6 +302,7 @@ class FileSourceFactory(ObjectFactory):
         self.length = 0
         self.thumbnail = ""
         self.thumbnails = []
+        self.settings = None
 
     def do_set_property(self, property, value):
         if property.name == "length":
@@ -223,7 +322,10 @@ class FileSourceFactory(ObjectFactory):
         bin = gst.Bin("%s-%d" % (self.name, self.lastbinid))
         self.lastbinid = self.lastbinid + 1
         src = gst.element_make_from_uri(gst.URI_SRC, self.name, "file source")
-        dbin = gst.element_factory_make("decodebin")
+        try:
+            dbin = gst.element_factory_make("decodebin2")
+        except:
+            dbin = gst.element_factory_make("decodebin")
         bin.add(src, dbin)
         src.link(dbin)
 
@@ -264,7 +366,7 @@ class FileSourceFactory(ObjectFactory):
         else:
             return
         bin.remove_pad(mypad)
-        
+
     def binIsDestroyed(self, bin):
         """ Remove the given bin from the list of instances """
         if bin in self.instances:
@@ -280,31 +382,50 @@ class FileSourceFactory(ObjectFactory):
 
     def getExportSettings(self):
         """ Returns the ExportSettings corresponding to this source """
-        settings = ExportSettings()
+        if self.settings:
+            return self.settings
+
+        self.settings = ExportSettings()
         if self.video_info_stream:
             # Fill video properties
             vs = self.video_info_stream
-            settings.videowidth = vs.width
-            settings.videoheight = vs.height
-            settings.videorate = vs.framerate
-            settings.videopar = vs.par
+            self.settings.videowidth = vs.width
+            self.settings.videoheight = vs.height
+            self.settings.videorate = vs.framerate
+            self.settings.videopar = vs.par
 
         if self.audio_info_stream:
             # Fill audio properties
             as = self.audio_info_stream
-            settings.audiochannels = as.channels
-            settings.audiorate = as.rate
-            settings.audiodepth = as.depth
-            
-        return settings
+            self.settings.audiochannels = as.channels
+            self.settings.audiorate = as.rate
+            self.settings.audiodepth = as.depth
+
+        return self.settings
+
+    # Serializable methods
+
+    def toDataFormat(self):
+        ret = ObjectFactory.toDataFormat(self)
+        ret["filename"] = self.name
+        ret["length"] = self.length
+        return ret
+
+    def fromDataFormat(self, obj):
+        ObjectFactory.fromDataFormat(self, obj)
+        self.name = obj["filename"]
+        self.length = obj["length"]
+
 
 class OperationFactory(ObjectFactory):
     """
     Provides operations useable in a timeline
     """
 
-    def __init__(self):
-        ObjectFactory.__init__(self)
+    __data_type__ = "operation-factory"
+
+    def __init__(self, **kwargs):
+        ObjectFactory.__init__(self, **kwargs)
         self.nbinput = 1
         self.nboutput = 1
 
@@ -314,9 +435,11 @@ class SimpleOperationFactory(OperationFactory):
     Provides simple (audio OR video) operations useable in a timeline
     """
 
-    def __init__(self, elementfactory):
+    __data_type__ = "simple-operation-factory"
+
+    def __init__(self, elementfactory, **kwargs):
         """ elementfactory is the GstElementFactory """
-        OperationFactory.__init__(self)
+        OperationFactory.__init__(self, **kwargs)
         self.name = elementfactory.get_name()
         self.displayname = elementfactory.get_longname()
         # check what type the output pad is (AUDIO/VIDEO)
@@ -333,8 +456,10 @@ class TransitionFactory(OperationFactory):
     Provides transitions useable in a timeline
     """
 
-    def __init__(self):
-        OperationFactory.__init__(self)
+    __data_type__ = "transition-factory"
+
+    def __init__(self, **kwargs):
+        OperationFactory.__init__(self, **kwargs)
 
 
 class SMPTETransitionFactory(TransitionFactory):
@@ -342,10 +467,26 @@ class SMPTETransitionFactory(TransitionFactory):
     Provides SMPTE video transitions useable in a timeline
     """
 
-    def __init__(self):
-        TransitionFactory.__init__(self)
+    __data_type__ = "SMPTE-transition-factory"
+
+    def __init__(self, **kwargs):
+        TransitionFactory.__init__(self, **kwargs)
+
+##
+## Multimedia streams, used for definition of media streams
+##
+
 
 class MultimediaStream:
+    """
+    Defines a media stream
+
+    Properties:
+    * raw (boolean) : True if the stream is a raw media format
+    * fixed (boolean) : True if the stream is entirely defined
+    * codec (string) : User-friendly description of the codec used
+    * caps (gst.Caps) : Caps corresponding to the stream
+    """
 
     def __init__(self, caps):
         gst.log("new with caps %s" % caps.to_string())
@@ -362,12 +503,22 @@ class MultimediaStream:
     def _analyzeCaps(self):
         raise NotImplementedError
 
+    def getMarkup(self):
+        """
+        Returns a pango-markup string definition of the stream
+        Subclasses need to implement this
+        """
+        raise NotImplementedError
+
 class VideoStream(MultimediaStream):
+    """
+    Video Stream
+    """
 
     def _analyzeCaps(self):
         if len(self.caps) > 1:
             self.fixed = False
-            
+
         struct = self.caps[0]
         self.videotype = struct.get_name()
         if self.videotype.startswith("video/x-raw-"):
@@ -418,8 +569,11 @@ class VideoStream(MultimediaStream):
                 templ = templ + _(" <i>(%s)</i>") % self.codec
             return templ
         return _("<b>Unknown Video format:</b> %s") % self.videotype
-            
+
 class AudioStream(MultimediaStream):
+    """
+    Audio stream
+    """
 
     def _analyzeCaps(self):
         if len(self.caps) > 1:
@@ -464,6 +618,9 @@ class AudioStream(MultimediaStream):
         return _("<b>Unknown Audio format:</b> %s") % self.audiotype
 
 class TextStream(MultimediaStream):
+    """
+    Text media stream
+    """
 
     def _analyzeCaps(self):
         if len(self.caps) > 1:
@@ -475,6 +632,10 @@ class TextStream(MultimediaStream):
         return _("<b>Text:</b> %s") % self.texttype
 
 def get_stream_for_caps(caps):
+    """
+    Returns the appropriate MediaStream corresponding to the
+    given caps.
+    """
     val = caps.to_string()
     if val.startswith("video/"):
         return VideoStream(caps)
@@ -483,5 +644,5 @@ def get_stream_for_caps(caps):
     elif val.startswith("text/"):
         return TextStream(caps)
     else:
+        # FIXME : we should have an 'unknow' data stream class
         return None
-                     
