@@ -118,7 +118,7 @@ class PlayGround(gobject.GObject):
         Adds the given pipeline to the playground.
         Returns True if the pipeline was added to the playground.
         """
-        gst.debug("pipeline : %s" % pipeline)
+        gst.debug("pipeline : %r" % pipeline)
         if not isinstance(pipeline, SmartBin):
             return False
 
@@ -135,7 +135,7 @@ class PlayGround(gobject.GObject):
         Removes the given pipeline from the playground.
         Return True if everything went well.
         """
-        gst.debug("pipeline : %s" % pipeline)
+        gst.debug("pipeline : %r" % pipeline)
         if not pipeline in self.pipelines:
             return True
 
@@ -216,7 +216,7 @@ class PlayGround(gobject.GObject):
 
     def setVideoSinkThread(self, vsinkthread):
         """ sets the video sink thread """
-        gst.debug("video sink thread : %s" % vsinkthread)
+        gst.debug("video sink thread : %r" % vsinkthread)
         if self.vsinkthread and self.current.has_video:
             self.current.set_state(gst.STATE_READY)
             self.current.removeVideoSinkThread()
@@ -226,7 +226,7 @@ class PlayGround(gobject.GObject):
 
     def setAudioSinkThread(self, asinkthread):
         """ sets the audio sink thread """
-        gst.debug("set audio sink thread : %s" % asinkthread)
+        gst.debug("set audio sink thread : %r" % asinkthread)
         if self.asinkthread and self.current.asinkthread:
             self.current.set_state(gst.STATE_READY)
             self.current.removeAudioSinkThread()
@@ -234,41 +234,56 @@ class PlayGround(gobject.GObject):
         if self.current and self.current.has_audio:
             self.current.setAudioSinkThread(self.asinkthread)
 
-    def _playTemporaryBin(self, tempbin):
+    def _playTemporaryBin(self, tempbin, playit=True):
         """
         Temporarely play a smartbin.
         Return False if there was a problem.
+
+        By default the bin will start playing, except if you set
+        playit to False
         """
-        gst.debug("BEGINNING tempbin : %s" % tempbin)
+        gst.debug("BEGINNING tempbin : %r" % tempbin)
         self.pause()
         self.addPipeline(tempbin)
         self.switchToPipeline(tempbin)
         if self.tempsmartbin:
             self.removePipeline(self.tempsmartbin)
         self.tempsmartbin = tempbin
-        if self.play() == gst.STATE_CHANGE_FAILURE:
+        if playit and self.play() == gst.STATE_CHANGE_FAILURE:
             return False
-        gst.debug("END tempbin : %s" % tempbin)
+        gst.debug("END tempbin : %r" % tempbin)
         return True
 
-    def playTemporaryFilesourcefactory(self, factory):
+    def playTemporaryFilesourcefactory(self, factory, playit=True):
         """
         Temporarely play a FileSourceFactory.
         Returns False if there was a problem.
+
+        By default the bin will start playing, except if you set
+        playit to False
         """
         gst.debug("factory : %s" % factory)
         if isinstance(self.current, SmartFileBin) and self.current.factory == factory:
             gst.info("Already playing factory : %s" % factory)
             return True
         tempbin = SmartFileBin(factory)
-        return self._playTemporaryBin(tempbin)
+        return self._playTemporaryBin(tempbin, playit)
 
-    def getCurrentTimePosition(self):
+    def getCurrentTimePosition(self, onlyvideo=True):
         """
         Returns the current position of the current bin in gst.FORMAT_TIME
         """
+        # FIXME : Yes, we're forcing by default to query the position only on
+        # the video stream. The problem is that because of some formats screwing
+        # up big time when doing accurate seek (namely mad/mp3 in avi), the
+        # position in the audiosink will be greater than the position in the
+        # videosink... and the result will be a position further than what's
+        # actually being displayed currently :(
         try:
-            cur, format = self.current.query_position(gst.FORMAT_TIME)
+            if onlyvideo and not self.current.recording:
+                cur,format = self.vsinkthread.query_position(gst.FORMAT_TIME)
+            else:
+                cur, format = self.current.query_position(gst.FORMAT_TIME)
         except:
             self.current.warning("Couldn't get position in time")
             cur = 0
@@ -340,7 +355,7 @@ class PlayGround(gobject.GObject):
     #
     def _busMessageCb(self, unused_bus, message, unused_pipeline):
         """ handler for messages from the pipelines' buses """
-        gst.info("%s [%s]" % (message.type, message.src))
+        gst.info("%s [%r]" % (message.type, message.src))
         if message.type == gst.MESSAGE_STATE_CHANGED:
             oldstate, newstate, pending = message.parse_state_changed()
             message.src.debug("old:%s, new:%s, pending:%s" %
@@ -401,7 +416,7 @@ class PlayGround(gobject.GObject):
         Uses the information from the Gerror, detail and source to
         create meaningful warning messages for the User Interface.
         """
-        gst.warning("gerror:%s , detail:%s , source:%s" % (gerror, detail, source))
+        gst.warning("gerror:%s , detail:%s , source:%r" % (gerror, detail, source))
         gst.warning("GError : code:%s, domain:%s, message:%s" % (gerror.code, gerror.domain, gerror.message))
 
 
