@@ -28,12 +28,15 @@ import gst
 
 from source import TimelineSource
 from objects import BrotherObjects, MEDIA_TYPE_AUDIO
+from pitivi.serializable import to_object_from_data_type
 
 class Layer(BrotherObjects):
     """
     Base class for composition layers (effects, sources, ...)
     """
 
+    __data_type__ = "timeline-layer"
+
     def __init__(self):
         gobject.GObject.__init__(self)
 
@@ -43,6 +46,8 @@ class EffectsLayer(Layer):
     Layers of the composition that have only one priority
     """
 
+    __data_type__ = "timeline-effects-layer"
+
     def __init__(self, priority):
         Layer.__init__(self)
         self._priority = priority
@@ -60,6 +65,8 @@ class SourcesLayer(Layer):
     Sources are sorted by start time and then by priority
     """
 
+    __data_type__ = "timeline-sources-layer"
+
     def __init__(self, minprio, maxprio):
         Layer.__init__(self)
         self._minprio = minprio
@@ -120,6 +127,14 @@ class TimelineComposition(TimelineSource):
       _ 'transition-removed' : a transitions was removed from the composition
       _ 'source-added' : a TimelineSource was added to the composition
       _ 'source-removed' : a TimelineSource was removed from the composition
+
+    Save/Load properties:
+    * (optional) 'global-effects' (list of TimelineSimpleEffect) : list of global effects
+    * (optional) 'simple-effects' (list of list of TimelineSimpleEffect) : list of list of simple effects
+    * (optional) 'complex-effects' (list of TimelineComplexEffect) : list of complex effects
+    * (optional) 'transitions' (list of TimelineTransition) : list of transitions
+    * 'sources' (list of list of sources) : layers of sources of the composition
+    * (optional) 'default-source' (TimelineSource) : Default source for the composition.
     """
 
     __gsignals__ = {
@@ -158,6 +173,9 @@ class TimelineComposition(TimelineSource):
                              (gobject.TYPE_PYOBJECT, )),
         }
 
+    __data_type__ = "timeline-composition"
+    __requires_factory__ = False
+
     # mid-level representation/storage of sources/effecst lists
     #
     # Global effects:
@@ -183,7 +201,7 @@ class TimelineComposition(TimelineSource):
 
     def __init__(self, **kw):
         self.global_effects = [] # list of effects starting from highest priority
-        self.simple_effects = [[]] # list of layers of simple effects (order: priority, then time)
+        self.simple_effects = [] # list of layers of simple effects (order: priority, then time)
         self.complex_effects = [] # complex effect sorted by time
         self.transitions = [] # transitions sorted by time
         # list of layers of simple effects (order: priority, then time)
@@ -193,6 +211,7 @@ class TimelineComposition(TimelineSource):
         self.sources = [(2048, 2060, [])]
         self.defaultSource = None
         TimelineSource.__init__(self, **kw)
+        self.setStartDurationTime(0,0)
 
     def __len__(self):
         """ return the number of sources in this composition """
@@ -310,12 +329,6 @@ class TimelineComposition(TimelineSource):
         if self.condensed:
             # compare it to the self.condensed
             list_changed = False
-##             print "comparing:"
-##             for i in self.condensed:
-##                 print i.gnlobject, i.start, i.duration
-##             print "with"
-##             for i in clist:
-##                 print i.gnlobject, i.start, i.duration
             if not len(clist) == len(self.condensed):
                 list_changed = True
             else:
@@ -359,7 +372,12 @@ class TimelineComposition(TimelineSource):
 
     # Sources
 
-    def _getSourcePosition(self, source):
+    def getSourceLayerPosition(self, source):
+        """
+        Returns the layer position of the given source in the timeline.
+        First position = 1
+        Not found = 0
+        """
         position = 0
         foundit = False
         for slist in self.sources:
@@ -371,6 +389,17 @@ class TimelineComposition(TimelineSource):
             return position + 1
         return 0
 
+    def getSimpleSourcePosition(self, source):
+        """
+        Returns the position of the given source in the timeline.
+        First position = 1
+        Not found = 0
+        """
+        layer = self.getSourceLayerPosition(source)
+        if not layer:
+            return 0
+        return self.sources[layer - 1][2].index(source) + 1
+
     def _haveGotThisSource(self, source):
         for slist in self.sources:
             if source in slist[2]:
@@ -420,6 +449,12 @@ class TimelineComposition(TimelineSource):
         """
         self.gnlobject.info("source %s , position:%d, self.sources:%s" %(source, position, self.sources))
 
+        # make sure object to add has valid start/duration
+        if source.start == -1 or source.duration <= 0:
+            self.gnlobject.warning("Trying to add a source with non-valid start/duration")
+            raise Exception("Source has invalid start[%s] or duration[%s]" % (gst.TIME_ARGS(source.start),
+                                                                              gst.TIME_ARGS(source.duration)))
+
         self._addSource(source, position)
 
         # if auto_linked and self.linked, add brother to self.linked with same parameters
@@ -444,15 +479,13 @@ class TimelineComposition(TimelineSource):
             existorder = 0
         else:
             start = existingsource.start + existingsource.duration
-            position = self._getSourcePosition(existingsource)
+            position = self.getSourceLayerPosition(existingsource)
             existorder = self.sources[position - 1][2].index(existingsource) + 1
 
         gst.info("start=%s, position=%d, existorder=%d, sourcelength=%s" % (gst.TIME_ARGS(start),
                                                                             position,
                                                                             existorder,
                                                                             gst.TIME_ARGS(source.factory.length)))
-##         for i in self.sources[position -1][2]:
-##             print i.gnlobject, i.start, i.duration
         # set the correct start/duration time
         duration = source.factory.length
         source.setStartDurationTime(start, duration)
@@ -460,12 +493,7 @@ class TimelineComposition(TimelineSource):
         # pushing following
         if push_following and not position in [-1, 0]:
             #print self.gnlobject, "pushing following", existorder, len(self.sources[position - 1][2])
-            for i in range(existorder, len(self.sources[position - 1][2])):
-                mvsrc = self.sources[position - 1][2][i]
-                self.gnlobject.info("pushing following")
-                #print "run", i, "start", mvsrc.start, "duration", mvsrc.duration
-                # increment self.sources[position - 1][i] by source.factory.length
-                mvsrc.setStartDurationTime(mvsrc.start + source.factory.length)
+            self.shiftSources(source.factory.length, existorder, len(self.sources[position - 1][2]))
 
         self.addSource(source, position, auto_linked=auto_linked)
 
@@ -534,10 +562,7 @@ class TimelineComposition(TimelineSource):
         # 1. if collapse_neighbours, shift all downstream sources by duration
         if collapse_neighbours and oldpos != len(sources) - 1:
             self.gnlobject.log("collapsing all following neighbours after the old position [%d]" % oldpos)
-            for i in range(oldpos + 1, len(sources)):
-                obj = sources[i]
-                self.gnlobject.log("moving source %d %s" % (i, obj))
-                obj.setStartDurationTime(start = (obj.start - source.duration))
+            self.shiftSources(-source.duration, oldpos + 1)
 
         # 2. if push_neighbours, make sure there's enough room at the new position
         if push_neighbours and newpos != len(sources):
@@ -608,14 +633,12 @@ class TimelineComposition(TimelineSource):
         # change it for the linked sources
         if collapse_neighbours:
             self.gnlobject.info("Collapsing neighbours")
-            for i in range(pos, len(sources[2])):
-                obj = sources[2][i]
-                obj.setStartDurationTime(start = (obj.start - source.duration))
+            self.shiftSources(-source.duration, pos)
 
         # if we have a brother
         if remove_linked and self.linked and source.linked:
             sources = self.linked.sources[0]
-            if source.linked in sources[2]: 
+            if source.linked in sources[2]:
                 pos = sources[2].index(source.linked)
                 del sources[2][pos]
                 self.linked.gnlobject.remove(source.linked.gnlobject)
@@ -626,16 +649,73 @@ class TimelineComposition(TimelineSource):
         # update the condensed list
         self._updateCondensedList()
 
+    def shiftSources(self, offset, startpos, endpos=-1):
+        """
+        Shifts by offset ns (can be negative) all sources from the given start
+        position to the given end position.
+        If no end position is specified (default:-1) all sources starting from
+        the given start position will be shifted.
+        """
+        self.gnlobject.info("offset:%d, startpos:%d, endpos:%d" % (offset, startpos, endpos))
+        sources = self.sources[0]
+        if endpos == -1:
+            endpos = len(sources[2])
+        if startpos > endpos:
+            raise Exception("startpos needs to be smaller or equal to endpos !")
+        for i in range(startpos, endpos):
+            obj = sources[2][i]
+            obj.gnlobject.log("Position %d , setting start from %d to %d" % (i, obj.start, obj.start + offset))
+            obj.setStartDurationTime(start = (obj.start + offset))
+
+    def shiftSourcesBySource(self, offset, startsource, endsource=None):
+        """
+        Shifts by offset ns (can be negative) all sources from the given start
+        position to the given end position.
+        If no end position is specified (default:-1) all sources starting from
+        the given start position will be shifted.
+        """
+        self.gnlobject.info("offset:%d, startsource:%s, endsource:%s" % (offset, startsource, endsource))
+        startpos = self.getSourcePosition(startsource)
+        if endsource:
+            endpos = self.getSourcePosition(endsource)
+        else:
+            endpos = -1
+        self.shiftSources(offset, startpos, endpos)
+
+    def cleanUp(self):
+        """
+        Removes all sources/effects from the composition
+        """
+        self.gnlobject.info("cleaning up")
+        # effects
+
+        for fx in self.global_effects:
+            self.removeGlobalEffect(fx)
+        for layer in self.simple_effects:
+            for fx in layer:
+                self.removeSimpleEffect(fx)
+        for fx in self.complex_effects:
+            self.removeComplexEffect(fx)
+        for fx in self.transitions:
+            self.removeTransition(fx)
+
+        # sources
+        for layer in self.sources:
+            min, max, sources = layer
+            for source in sources[:]:
+                self.removeSource(source)
+
+    # Default Source methods
 
     def setDefaultSource(self, source):
         """
-        Adds a default source to the composition.
+        Adds a default TimelineSource to the composition.
         Default sources will be used for gaps within the composition.
         """
         if self.defaultSource:
-            self.gnlobject.remove(self.defaultSource)
-        source.props.priority = 2 ** 32 - 1
-        self.gnlobject.add(source)
+            self.gnlobject.remove(self.defaultSource.gnlobject)
+        source.gnlobject.props.priority = 2 ** 32 - 1
+        self.gnlobject.add(source.gnlobject)
         self.defaultSource = source
 
     def getDefaultSource(self):
@@ -693,3 +773,98 @@ class TimelineComposition(TimelineSource):
                 return self._autoAudioSettings()
             else:
                 return self._autoVideoSettings()
+
+    # Serializable methods
+
+    def toDataFormat(self):
+        ret = TimelineSource.toDataFormat(self)
+
+        # effects (optional)
+        if self.global_effects:
+            ret["global-effects"] = [fx.toDataFormat() for fx in self.global_effects]
+        if self.simple_effects and len(self.simple_effects[0]):
+            ret["simple-effects"] = [[fx.toDataFormat() for fx in ls] for ls in self.simple_effects]
+        if self.complex_effects:
+            ret["complex-effects"] = [fx.toDataFormat() for fx in self.complex_effects]
+        if self.transitions:
+            ret["transitions"] = [fx.toDataFormat() for fx in self.transitions]
+
+        # sources
+        # WARNING / FIXME / TODO : This is a temporary format !!!
+        tmp = []
+        for layer in self.sources:
+            lay = []
+            for source in layer[2]:
+                lay.append(source.toDataFormat())
+            tmp.append(lay)
+        ret["sources"] = tmp
+
+        # default source
+        if self.defaultSource:
+            ret["default-source"] = self.defaultSource.toDataFormat()
+        return ret
+
+    def fromDataFormat(self, obj):
+        TimelineSource.fromDataFormat(self, obj)
+
+        # deserialize all objects
+        globalfx = []
+        simplefx = []
+        complexfx = []
+        transitions = []
+        layers = []
+
+        if "global-effects" in obj:
+            for fx in obj["global-effects"]:
+                globalfx.append(to_object_from_data_type(fx))
+                #self.global_effects.append(to_object_from_data_type(fx))
+        if "simple-effects" in obj:
+            for line in obj["simple-effects"]:
+                tmp = []
+                for fx in line:
+                    tmp.append(to_object_from_data_type(fx))
+                simplefx.append(tmp)
+                #self.simple_effects.append(tmp)
+        if "complex-effects" in obj:
+            for fx in obj["complex-effects"]:
+                complexfx.append(to_object_from_data_type(fx))
+                #self.complex_effects.append(to_object_from_data_type(fx))
+        if "transitions" in obj:
+            for fx in obj["transitions"]:
+                transitions.append(to_object_from_data_type(fx))
+                #self.transitions.append(to_object_from_data_type(fx))
+
+        # WARNING / FIXME / TODO : This is a temporary format !!!
+        gst.log("recreating sources")
+        pos = 1
+        for layer in obj["sources"]:
+            tmp = []
+            for source in layer:
+                tmp.append(to_object_from_data_type(source))
+            layers.append(tmp)
+            pos += 1
+
+        gst.log("adding effects")
+        # add objects
+        for fx in globalfx:
+            self.global_effects.append(fx)
+        for lay in simplefx:
+            self.simple_effects.append(lay)
+        for fx in complexfx:
+            self.complex_effects.append(fx)
+        for fx in transitions:
+            self.transitions.append(fx)
+
+        gst.log("adding sources")
+        pos = 1
+        for layer in layers:
+            for source in layer:
+                self.addSource(source, pos, auto_linked=False)
+            pos +=1
+
+        # default source
+        if "default-source" in obj:
+            self.setDefaultSource(to_object_from_data_type(obj["default-source"]))
+        gst.log("done adding sources")
+
+        self._updateCondensedList()
