@@ -38,6 +38,7 @@ else:
 
 import pitivi.instance as instance
 import pitivi.configure as configure
+from pitivi.projectsaver import ProjectSaver
 
 from gettext import gettext as _
 
@@ -46,6 +47,7 @@ from sourcefactories import SourceFactoriesWidget
 from viewer import PitiviViewer
 from pitivi.bin import SmartTimelineBin
 from projectsettings import ProjectSettingsDialog
+from pluginmanagerdialog import PluginManagerDialog
 from pitivi.configure import pitivi_version, APPNAME
 from glade import GladeWindow
 
@@ -72,13 +74,19 @@ class PitiviMainWindow(gtk.Window):
         self._createStockIcons()
         self._setActions()
         self._createUi()
+        self.actions = None
+        self.toggleactions = None
+        self.actiongroup = None
 
         self.isFullScreen = False
         self.errorDialogBox = None
 
-        instance.PiTiVi.connect("new-project", self._newProjectCb)
+        instance.PiTiVi.connect("new-project-loaded", self._newProjectLoadedCb)
+        instance.PiTiVi.connect("new-project-loading", self._newProjectLoadingCb)
         instance.PiTiVi.connect("closing-project", self._closingProjectCb)
-        instance.PiTiVi.connect("not-project", self._notProjectCb)
+        instance.PiTiVi.connect("new-project-failed", self._notProjectCb)
+        instance.PiTiVi.current.connect("save-uri-requested", self._saveAsDialogCb)
+        instance.PiTiVi.current.connect("confirm-overwrite", self._confirmOverwriteCb)
         instance.PiTiVi.playground.connect("error", self._playGroundErrorCb)
         instance.PiTiVi.current.sources.connect_after("file_added", self._sourcesFileAddedCb)
 
@@ -98,6 +106,8 @@ class PitiviMainWindow(gtk.Window):
 
     def _timelineDurationChangedCb(self, unused_composition, unused_start,
                                    duration):
+        if not isinstance(instance.PiTiVi.playground.current, SmartTimelineBin):
+            return
         self.render_button.set_sensitive((duration > 0) and True or False)
         if duration > 0 :
             gobject.idle_add(self.timeline.simpleview._displayTimeline)
@@ -107,7 +117,8 @@ class PitiviMainWindow(gtk.Window):
 	    self.render_button.set_sensitive(False)
         else:
             if isinstance(smartbin, SmartTimelineBin):
-                gst.info("switching to Timeline, setting duration to %s" % (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
+                gst.info("switching to Timeline, setting duration to %s" %
+                         (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
                 smartbin.project.timeline.videocomp.connect("start-duration-changed",
                                                             self._timelineDurationChangedCb)
                 if smartbin.project.timeline.videocomp.duration > 0:
@@ -125,10 +136,12 @@ class PitiviMainWindow(gtk.Window):
                 ('pitivi-render', 'Render', 0, 0, 'pitivi')
                 ])
         factory = gtk.IconFactory()
-        pixbuf = gtk.gdk.pixbuf_new_from_file (configure.get_pixmap_dir() + "/pitivi-advanced-24.png")
+        pixbuf = gtk.gdk.pixbuf_new_from_file(
+            configure.get_pixmap_dir() + "/pitivi-advanced-24.png")
         iconset = gtk.IconSet(pixbuf)
         factory.add('pitivi-advanced-mode', iconset)
-        pixbuf = gtk.gdk.pixbuf_new_from_file (configure.get_pixmap_dir() + "/pitivi-render-24.png")
+        pixbuf = gtk.gdk.pixbuf_new_from_file(
+            configure.get_pixmap_dir() + "/pitivi-render-24.png")
         iconset = gtk.IconSet(pixbuf)
         factory.add('pitivi-render', iconset)
         factory.add_default()
@@ -137,25 +150,41 @@ class PitiviMainWindow(gtk.Window):
     def _setActions(self):
         """ sets up the GtkActions """
         self.actions = [
-            ("NewProject", gtk.STOCK_NEW, None, None, _("Create a new project"), self._newProjectMenuCb),
-            ("OpenProject", gtk.STOCK_OPEN, None, None, _("Open an existing project"), self._openProjectCb),
-            ("SaveProject", gtk.STOCK_SAVE, None, None, _("Save the current project"), self._saveProjectCb),
-            ("SaveProjectAs", gtk.STOCK_SAVE_AS, None, None, _("Save the current project"), self._saveProjectAsCb),
-            ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project settings"), None, _("Edit the project settings"), self._projectSettingsCb),
-            ("ImportSources", gtk.STOCK_ADD, _("_Import clips..."), None, _("Import clips to use"), self._importSourcesCb),
-            ("ImportSourcesFolder", gtk.STOCK_ADD, _("_Import folder of clips..."), None, _("Import folder of clips to use"), self._importSourcesFolderCb),
-	    ("RenderProject", 'pitivi-render' , _("_Render project"), None, _("Render project"), self._recordCb),
+            ("NewProject", gtk.STOCK_NEW, None,
+             None, _("Create a new project"), self._newProjectMenuCb),
+            ("OpenProject", gtk.STOCK_OPEN, None,
+             None, _("Open an existing project"), self._openProjectCb),
+            ("SaveProject", gtk.STOCK_SAVE, None,
+             None, _("Save the current project"), self._saveProjectCb),
+            ("SaveProjectAs", gtk.STOCK_SAVE_AS, None,
+             None, _("Save the current project"), self._saveProjectAsCb),
+            ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project settings"),
+             None, _("Edit the project settings"), self._projectSettingsCb),
+            ("ImportSources", gtk.STOCK_ADD, _("_Import clips..."),
+             None, _("Import clips to use"), self._importSourcesCb),
+            ("ImportSourcesFolder", gtk.STOCK_ADD,
+             _("_Import folder of clips..."), None,
+             _("Import folder of clips to use"), self._importSourcesFolderCb),
+            ("RenderProject", 'pitivi-render' , _("_Render project"),
+             None, _("Render project"), self._recordCb),
+            ("PluginManager", gtk.STOCK_PREFERENCES ,
+             _("_Plugins..."),
+             None, _("Manage plugins"), self._pluginManagerCb),
             ("Quit", gtk.STOCK_QUIT, None, None, None, self._quitCb),
-            ("About", gtk.STOCK_ABOUT, None, None, _("Information about %s") % APPNAME, self._aboutCb),
+            ("About", gtk.STOCK_ABOUT, None, None,
+             _("Information about %s") % APPNAME, self._aboutCb),
             ("File", None, _("_File")),
+            ("Edit", None, _("_Edit")),
             ("View", None, _("_View")),
             ("Help", None, _("_Help"))
-            ]
+        ]
 
         self.toggleactions = [
-            ("AdvancedView", 'pitivi-advanced-mode', _("Advanced vie_w"), None, _("Switch to advanced view"), self._advancedViewCb),
-            ("FullScreen", gtk.STOCK_FULLSCREEN, None, None, _("View the main window on the whole screen"), self._fullScreenCb)
-            ]
+            ("AdvancedView", 'pitivi-advanced-mode', _("Advanced vie_w"),
+             None, _("Switch to advanced view"), self._advancedViewCb),
+            ("FullScreen", gtk.STOCK_FULLSCREEN, None, None,
+             _("View the main window on the whole screen"), self._fullScreenCb)
+        ]
 
         self.actiongroup = gtk.ActionGroup("mainwindow")
         self.actiongroup.add_actions(self.actions)
@@ -166,13 +195,18 @@ class PitiviMainWindow(gtk.Window):
         for action in self.actiongroup.list_actions():
             if action.get_name() == "RenderProject":
                 self.render_button = action
-            if action.get_name() == "AdvancedView":
+            elif action.get_name() == "AdvancedView":
                 if not instance.PiTiVi.settings.advancedModeEnabled:
                     action.set_visible(False)
-            if action.get_name() in ["ProjectSettings", "Quit", "File", "Help",
-                                     "About", "View", "FullScreen", "ImportSources",
-                                     "ImportSourcesFolder", "AdvancedView"]:
+            elif action.get_name() in [
+                "ProjectSettings", "Quit", "File", "Edit", "Help",
+                "About", "View", "FullScreen", "ImportSources",
+                "ImportSourcesFolder", "AdvancedView", "PluginManager"]:
                 action.set_sensitive(True)
+            elif action.get_name() in ["SaveProject", "SaveProjectAs",
+                    "NewProject", "OpenProject"]:
+                if not instance.PiTiVi.settings.fileSupportEnabled:
+                    action.set_sensitive(False)
             else:
                 action.set_sensitive(False)
 
@@ -277,16 +311,40 @@ class PitiviMainWindow(gtk.Window):
     ## Toolbar/Menu actions callback
 
     def _newProjectMenuCb(self, unused_action):
-        instance.PiTiVi.new_blank_project()
+        instance.PiTiVi.newBlankProject()
 
     def _openProjectCb(self, unused_action):
-        raise NotImplementedError
+        chooser = gtk.FileChooserDialog(_("Open File ..."),
+            self,
+            action=gtk.FILE_CHOOSER_ACTION_OPEN,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        chooser.set_select_multiple(False)
+        formats = ProjectSaver.listFormats()
+        default = gtk.FileFilter()
+        default.set_name("All")
+        default.add_pattern("*")
+        chooser.add_filter(default)
+        for format in formats:
+            filt = gtk.FileFilter()
+            filt.set_name(format[0])
+            for ext in format[1]:
+                filt.add_pattern("*%s" % ext)
+            chooser.add_filter(filt)
+
+        response = chooser.run()
+
+        if response == gtk.RESPONSE_OK:
+            path = chooser.get_filename()
+            instance.PiTiVi.loadProject(filepath = path)
+        chooser.destroy()
+        return True
 
     def _saveProjectCb(self, unused_action):
-        raise NotImplementedError
+        instance.PiTiVi.current.save()
 
     def _saveProjectAsCb(self, unused_action):
-        raise NotImplementedError
+        instance.PiTiVi.current.saveAs()
 
     def _projectSettingsCb(self, unused_action):
         l = ProjectSettingsDialog(self, instance.PiTiVi.current)
@@ -312,7 +370,7 @@ class PitiviMainWindow(gtk.Window):
         abt.set_name(APPNAME)
         abt.set_version("v%s" % pitivi_version)
         abt.set_website("http://www.pitivi.org/")
-        authors = ["Edward Hervey <edward@fluendo.com>","",_("Contributors:"),
+        authors = ["Edward Hervey <bilboed@bilboed.com>","",_("Contributors:"),
                    "Christophe Sauthier <christophe.sauthier@gmail.com> (i18n)",
                    "Laszlo Pandy <laszlok2@gmail.com> (UI)",
                    "Ernst Persson  <ernstp@gmail.com>",
@@ -320,7 +378,9 @@ class PitiviMainWindow(gtk.Window):
                    "Thibaut Girka <thibaut.girka@free.fr> (UI)",
                    "Jeff Fortin <nekohayo@gmail.com> (UI)",
                    "Johan Dahlin <jdahlin@async.com.br> (UI)",
-                   "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)"]
+                   "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)",
+                   "Luca Della Santina <dellasantina@farm.unipi.it>",
+                   "Thijs Vermeir <thijsvermeir@gmail.com>"]
         abt.set_authors(authors)
         abt.set_license(_("GNU Lesser General Public License\nSee http://www.gnu.org/copyleft/lesser.html for more details"))
         abt.set_icon_from_file(configure.get_global_pixmap_dir() + "/pitivi.png")
@@ -333,21 +393,106 @@ class PitiviMainWindow(gtk.Window):
     def _importSourcesFolderCb(self, unused_action):
         self.sourcefactories.sourcelist.showImportSourcesDialog(True)
 
+    def _pluginManagerCb(self, unused_action):
+        PluginManagerDialog(instance.PiTiVi.plugin_manager)
+
     ## PiTiVi main object callbacks
 
-    def _newProjectCb(self, pitivi, project):
-        raise NotImplementedError
+    def _newProjectLoadedCb(self, pitivi, project):
+        gst.log("A NEW project is loaded, update the UI!")
+        # ungrey UI
+        self.set_sensitive(True)
+
+    def _newProjectLoadingCb(self, pitivi, project):
+        gst.log("A NEW project is being loaded, deactivate UI")
+        # grey UI
+        self.set_sensitive(False)
 
     def _closingProjectCb(self, pitivi, project):
-        # Return True if we accept the project being close
-        # if we want to save it before it being closed, we must
-        #   do so
+        if not project.hasUnsavedModifications():
+            return True
 
-        # For the time being we always accept it being closed
-        return True
+        dialog = gtk.MessageDialog(
+            self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_QUESTION,
+            gtk.BUTTONS_YES_NO,
+            _("The project has unsaved changes. Do you wish to close the project?"))
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_YES:
+            return True
+        return False
+
+    def _notProjectCb(self, pitivi, reason, uri):
+        # ungrey UI
+        dialog = gtk.MessageDialog(self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_ERROR,
+            gtk.BUTTONS_OK,
+            _("PiTiVi is unable to load file \"%s\": %s") %
+                (uri, reason))
+        dialog.set_title(_("Error Loading File"))
+        dialog.run()
+        dialog.destroy()
+        self.set_sensitive(True)
+
+    ## PiTiVi current project callbacks
+
+    def _confirmOverwriteCb(self, unused_project, uri):
+        message = _("Do you wish to overwrite existing file \"%s\"?") %\
+                 gst.uri_get_location(uri)
+
+        dialog = gtk.MessageDialog(self,
+            gtk.DIALOG_MODAL,
+            gtk.MESSAGE_WARNING,
+            gtk.BUTTONS_YES_NO,
+            message)
+
+        dialog.set_title(_("Overwrite Existing File?"))
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_YES:
+            return True
+        return False
+
+    def _saveAsDialogCb(self, project):
+        chooser = gtk.FileChooserDialog(_("Save As..."),
+            self,
+            action=gtk.FILE_CHOOSER_ACTION_SAVE,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+            gtk.STOCK_SAVE, gtk.RESPONSE_OK))
+
+        chooser.set_select_multiple(False)
+        chooser.set_current_name(_("Untitled.pptv"))
+        chooser.set_default_response(gtk.RESPONSE_CANCEL)
+        formats = ProjectSaver.listFormats()
+        default = gtk.FileFilter()
+        default.set_name(_("Detect Automatically"))
+        default.add_pattern("*")
+        chooser.add_filter(default)
+        for format in formats:
+            filt = gtk.FileFilter()
+            filt.set_name(format[0])
+            for ext in format[1]:
+                filt.add_pattern("*.%s" % ext)
+            chooser.add_filter(filt)
+
+        response = chooser.run()
+
+        if response == gtk.RESPONSE_OK:
+            # need to do this to work around bug in gst.uri_construct
+            # which escapes all /'s in path!
+            uri = "file://" + chooser.get_filename()
+            format = chooser.get_filter().get_name()
+            if format == _("Detect Automatically"):
+                format = None
+            project.setUri(uri, format)
+            chooser.destroy()
+            return True
+        chooser.destroy()
+        return False
 
-    def _notProjectCb(self, pitivi, uri):
-        raise NotImplementedError
 
 class EncodingDialog(GladeWindow):
     """ Encoding dialog box """
@@ -398,9 +543,10 @@ class EncodingDialog(GladeWindow):
 
     def _positionCb(self, unused_playground, unused_smartbin, position):
         timediff = time.time() - self.timestarted
-        self.progressbar.set_fraction(float(position) / float(self.bin.length))
-        if timediff > 5.0:
-            # only display ETA after 5s in order to have enough averaging
+        self.progressbar.set_fraction(float(min(position, self.bin.length)) / float(self.bin.length))
+        if timediff > 5.0 and position:
+            # only display ETA after 5s in order to have enough averaging and
+            # if the position is non-null
             totaltime = (timediff * float(self.bin.length) / float(position)) - timediff
             self.progressbar.set_text(_("Finished in %dm%ds") % (int(totaltime) / 60,
                                                               int(totaltime) % 60))
