@@ -38,13 +38,14 @@ from gettext import gettext as _
 
 def time_to_string(value):
     if value == -1:
-        return "--m--s---"
+        return "--:--:--.---"
     ms = value / gst.MSECOND
     sec = ms / 1000
     ms = ms % 1000
     mins = sec / 60
     sec = sec % 60
-    return "%02dm%02ds%03d" % (mins, sec, ms)
+    hours = mins / 60
+    return "%02d:%02d:%02d.%03d" % (hours, mins, sec, ms)
 
 class PitiviViewer(gtk.VBox):
     """ Pitivi's viewer widget with controls """
@@ -64,13 +65,16 @@ class PitiviViewer(gtk.VBox):
         # changing project.
         self.project_signals = SignalGroup()
         self._connectToProject(instance.PiTiVi.current)
-        instance.PiTiVi.connect("new-project", self._newProjectCb)
+        instance.PiTiVi.connect("new-project-loaded", self._newProjectCb)
         instance.PiTiVi.playground.connect("current-state", self._currentStateCb)
         instance.PiTiVi.playground.connect("position", self._playgroundPositionCb)
 
         # callback to know when to set the XID on our viewer widget
         instance.PiTiVi.playground.connect("element-message", self._playgroundElementMessageCb)
 
+        # signal for timeline duration changes : (composition, sigid)
+        self._timelineDurationChangedSigId = (None, None)
+
         self._addTimelineToPlayground()
 
     def _connectToProject(self, project):
@@ -284,7 +288,7 @@ class PitiviViewer(gtk.VBox):
         if not self.currentlySeeking:
             self.currentlySeeking = True
             gobject.timeout_add(80, self._seekTimeoutCb)
-            instance.PiTiVi.playground.seekInCurrent(value, format)
+            instance.PiTiVi.playground.seekInCurrent(value, format=format)
             self._newTime(value)
         if format == gst.FORMAT_TIME:
             self.requested_time = value
@@ -302,7 +306,8 @@ class PitiviViewer(gtk.VBox):
     ## active Timeline calllbacks
 
     def _asyncTimelineDurationChanged(self, duration):
-        gst.debug("duration : %d" % duration)
+        gst.debug("duration : %s" % gst.TIME_ARGS(duration))
+        gst.debug("playground.current.length : %s" % gst.TIME_ARGS(instance.PiTiVi.playground.current.length))
         self.posadjust.upper = float(duration)
         self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time), time_to_string(instance.PiTiVi.playground.current.length)))
 
@@ -314,7 +319,7 @@ class PitiviViewer(gtk.VBox):
 
     def _timelineDurationChangedCb(self, unused_composition, unused_start,
                                    duration):
-        gst.debug("duration : %d" % duration)
+        gst.debug("duration : %s" % gst.TIME_ARGS(duration))
         if duration == 0:
             gobject.idle_add(self._backToDefaultCb)
         gobject.idle_add(self._asyncTimelineDurationChanged, duration)
@@ -386,14 +391,25 @@ class PitiviViewer(gtk.VBox):
             self.playpause_button.set_sensitive(False)
             self.next_button.set_sensitive(False)
             self.back_button.set_sensitive(False)
+            if not self._timelineDurationChangedSigId == (None, None):
+                obj, sigid = self._timelineDurationChangedSigId
+                obj.disconnect(sigid)
+                self._timelineDurationChangedSigId = (None, None)
         else:
             if isinstance(smartbin, SmartTimelineBin):
                 gst.info("switching to Timeline, setting duration to %s" % (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
                 self.posadjust.upper = float(smartbin.project.timeline.videocomp.duration)
-                smartbin.project.timeline.videocomp.connect("start-duration-changed",
-                                                            self._timelineDurationChangedCb)
+                # FIXME : we need to disconnect from this signal !
+                sigid = smartbin.project.timeline.videocomp.connect("start-duration-changed",
+                                                                    self._timelineDurationChangedCb)
+                self._timelineDurationChangedSigId = (smartbin.project.timeline.videocomp,
+                                                      sigid)
             else:
                 self.posadjust.upper = float(smartbin.factory.length)
+                if not self._timelineDurationChangedSigId == (None, None):
+                    obj, sigid = self._timelineDurationChangedSigId
+                    obj.disconnect(sigid)
+                    self._timelineDurationChangedSigId = (None, None)
             self._newTime(0)
             self.slider.set_sensitive(True)
             self.playpause_button.set_sensitive(True)
@@ -415,7 +431,8 @@ class PitiviViewer(gtk.VBox):
             gst.log('%s' % dav)
             if dav and dav.realsink and dav.realsink == message.src:
                 self.drawingarea.can_set_xid = True
-                self.drawingarea.set_xwindow_id()
+                if self.drawingarea.isexposed:
+                    self.drawingarea.set_xwindow_id()
 
 
 class ViewerWidget(gtk.DrawingArea):
@@ -432,9 +449,12 @@ class ViewerWidget(gtk.DrawingArea):
         self.can_set_xid = False
         self.unset_flags(gtk.DOUBLE_BUFFERED)
         self.unset_flags(gtk.SENSITIVE)
+        self.isexposed = False
 
     def do_expose_event(self, unused_event):
         """ 'expose-event' override """
+        gst.log("expose have_set_xid:%d can_set_xid:%d" % (self.have_set_xid, self.can_set_xid))
+        self.isexposed = True
         if self.videosink:
             if not self.have_set_xid and self.can_set_xid:
                 self.set_xwindow_id()
@@ -453,7 +473,7 @@ class ViewerWidget(gtk.DrawingArea):
                                                  self.window.xid))
         self.videosink.set_xwindow_id(self.window.xid)
         self.have_set_xid = True
-        self.videosink.expose()
+        #self.videosink.expose()
 
 
 class PlayPauseButton(gtk.Button):
